import { useState } from "react";
import { AnimatePresence } from "motion/react";
import { useTranslation } from "react-i18next";
import { useToast } from "../hooks/useToast";
import { useLoading } from "../hooks/useLoading";
import FormField from "../components/FormField";
import FormInput from "../components/FormInput";
import FormSelect from "../components/FormSelect";
import Button from "../components/Button";
import { lockerService } from "../services/locker.service";
import FormBox from "../components/FormBox";
import Notification from "../components/Notification";
import FieldsWrapper from "../components/FieldsWrapper";

function KeySwap() {
  const { t } = useTranslation();
  const showToast = useToast();
  const { isLoading, setIsLoading } = useLoading();
  const [notification, setNotification] = useState({ message: "", type: "" });
  const [keySwapDetails, setKeySwapDetails] = useState({
    cabinetId: "",
    lockerId: "",
    reason: "",
    oldKey: "",
    newKey: "",
    newKeyConfirm: "",
    cabinetIdValid: true,
    lockerIdValid: true,
    reasonValid: true,
    oldKeyValid: true,
    newKeyValid: true,
    newKeyConfirmValid: true,
  });

  const formFields = [
    {
      name: "cabinetId",
      label: t("cabinetId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "lockerId",
      label: t("lockerId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "reason",
      label: t("reasonForChange"),
      type: "input",
      maxLength: 50,
      readOnly: isLoading,
      validate: (value) => value !== "",
    },
    {
      name: "oldKey",
      label: t("oldKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "newKey",
      label: t("newKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => /^[A-Z]{2}[0-9]{4}/.test(value),
    },
    {
      name: "newKeyConfirm",
      label: t("confirmNewKeyId"),
      type: "input",
      maxLength: 6,
      readOnly: isLoading,
      validate: (value) => value !== "" && value === keySwapDetails.newKey,
    },
  ];

  const handleKeySwap = async (e) => {
    e.preventDefault();
    let valid = true;

    for (const field of formFields) {
      if (!field.validate(keySwapDetails[field.name])) {
        valid = false;
        setKeySwapDetails((prev) => ({
          ...prev,
          [`${field.name}Valid`]: false,
        }));
      }
    }

    if (!valid) {
      showToast(t("highlightedFieldsInvalid"), "error");
      return;
    }

    setIsLoading(true);
    try {
      const response = await lockerService.keySwap(
        keySwapDetails.cabinetId,
        keySwapDetails.lockerId,
        keySwapDetails.reason,
        keySwapDetails.oldKey,
        keySwapDetails.newKey
      );
      if (response.status === 200) {
        setNotification({
          message: response.data.message,
          type: "success",
        });
      } else {
        setNotification({
          message: response.data.message,
          type: "error",
        });
      }
    } catch (error) {
      setNotification({
        message: error.message,
        type: "error",
      });
    } finally {
      setIsLoading(false);
    }
  };

  const renderField = (field) => {
    const commonProps = {
      value: keySwapDetails[field.name],
      onChange: (e) => {
        const newLockerDetails = { ...keySwapDetails };
        newLockerDetails[field.name] = e.target.value.toUpperCase();
        newLockerDetails[`${field.name}Valid`] = true;
        setKeySwapDetails(newLockerDetails);
      },
      maxLength: field.maxLength,
      type: field.subType,
      readOnly: field.readOnly,
    };
    const valid = keySwapDetails[`${field.name}Valid`];
    return (
      <FormField key={field.name} label={field.label} icon={field.icon}>
        {field.type === "input" ? (
          <FormInput props={commonProps} valid={valid} />
        ) : (
          <FormSelect
            props={commonProps}
            valid={valid}
            options={field.options}
          />
        )}
      </FormField>
    );
  };

  return (
    <div>
      <AnimatePresence>
        {notification.message !== "" && <Notification {...notification} />}
      </AnimatePresence>
      <FormBox title={t("lockerStatus")}>
        <FieldsWrapper>{formFields.map(renderField)}</FieldsWrapper>
        <Button text={t("submit")} onClick={handleKeySwap} />
      </FormBox>
    </div>
  );
}

export default KeySwap;
