import { useLocation } from "react-router-dom";
import { useState } from "react";
import { useLoading } from "../hooks/useLoading";
import FormBox from "../components/FormBox";
import Button from "../components/Button";
import { lockerService } from "../services/locker.service";
import { AnimatePresence } from "motion/react";
import Notification from "../components/Notification";
import FormField from "../components/FormField";
import FormInput from "../components/FormInput";
import FormSelect from "../components/FormSelect";
import FormHeader from "../components/FormHeader";
import FieldsWrapper from "../components/FieldsWrapper";

function LockersRegistration() {
  const location = useLocation();
  const { setIsLoading } = useLoading();
  const [notification, setNotification] = useState({ message: "", type: "" });

  const noOfLockers = location.state?.noOfLockers;
  const cabinetId = location.state?.cabinetId;

  const initLockers = Array(noOfLockers)
    .fill()
    .map(() => ({
      id: "",
      size: "",
      keyId: "",
      idValid: true,
      sizeValid: true,
      keyIdValid: true,
    }));

  const [lockerValues, setLockerValues] = useState(initLockers);

  if (!location.state) {
    return <></>;
  }

  const handleSubmit = async (e) => {
    e.preventDefault();
    const idRegex = /^[A-Z]{2}[0-9]{4}$/;
    let valid = true;

    // Helper function to find duplicates
    const findDuplicates = (arr, key) => {
      const values = arr.map((item) => item[key]);
      return values.filter((item, index) => values.indexOf(item) !== index);
    };

    // Find duplicates
    const duplicateLockerIds = findDuplicates(lockerValues, "id");
    const duplicateKeyIds = findDuplicates(lockerValues, "keyId");

    const newValues = lockerValues.map((locker) => {
      const newLocker = { ...locker };

      // Check ID
      if (
        locker.id === "" ||
        !idRegex.test(locker.id) ||
        duplicateLockerIds.includes(locker.id)
      ) {
        newLocker.idValid = false;
        valid = false;
      } else {
        newLocker.idValid = true;
      }

      // Check size
      if (locker.size === "") {
        newLocker.sizeValid = false;
        valid = false;
      } else {
        newLocker.sizeValid = true;
      }

      // Check keyId
      if (
        locker.keyId === "" ||
        !idRegex.test(locker.keyId) ||
        duplicateKeyIds.includes(locker.keyId)
      ) {
        newLocker.keyIdValid = false;
        valid = false;
      } else {
        newLocker.keyIdValid = true;
      }

      return newLocker;
    });

    setLockerValues(newValues);

    if (!valid) {
      return;
    }

    try {
      setIsLoading(true);
      const response = await lockerService.registerLockers(
        cabinetId,
        lockerValues
      );
      setNotification({
        message: `Cabinet creation successful. Cabinet ID: ${response.data.cabinetId}`,
        type: "success",
      });
    } catch (error) {
      console.error(error);
      setNotification({
        message: `Error registering lockers. ${error.message}`,
        type: "error",
      });
      return;
    } finally {
      setIsLoading(false);
    }
  };

  const formRow = [
    {
      label: "Locker ID",
      type: "input",
      subType: "text",
      name: "id",
      maxLength: 6,
      validate: (value) => /^[A-Z]{2}[0-9]{4}$/.test(value),
    },
    {
      label: "Size",
      type: "select",
      subType: "text",
      name: "size",
      options: [
        { label: "Small", value: "1" },
        { label: "Medium", value: "2" },
        { label: "Large", value: "3" },
      ],
    },
    {
      label: "Key ID",
      type: "input",
      subType: "text",
      name: "keyId",
      maxLength: 6,
      validate: (value) => /^[A-Z]{2}[0-9]{4}$/.test(value),
    },
  ];
  const renderFormRow = (field) => {
    const commonProps = {
      value: lockerValues[field.name],
      onChange: (e) => {
        const newLockerValues = [...lockerValues];
        newLockerValues[field.name] = e.target.value;
        newLockerValues[`${field.name}Valid`] = true;
        setLockerValues(newLockerValues);
      },
      maxLength: field.maxLength,
      type: field.subType,
    };

    const valid = lockerValues[`${field.name}Valid`];

    return field.type === "input" ? (
      <FormInput key={field.name} props={commonProps} valid={valid} />
    ) : (
      <FormSelect key={field.name} props={commonProps} valid={valid} options={field.options} />
    );
  };

  const formFields = Array(noOfLockers).fill(formRow);
  const renderFormFields = (row, index) => {
    return (
      <FormField key={index} label={`Locker ${index + 1}`} variant="long">
        {row.map(renderFormRow)}
      </FormField>
    );
  };

  return (
    <div>
      <AnimatePresence>
        {notification.message !== "" && <Notification {...notification} />}
      </AnimatePresence>
      <div className="relative">
        {notification.type === "success" && (
          <div className="absolute inset-0 bg-[#fff]/50 z-10 rounded-3xl" />
        )}
        <FormBox title="Locker Registration">
          <FormHeader text={cabinetId} />
          <FieldsWrapper className="">
            {formFields.map(renderFormFields)}
          </FieldsWrapper>
          <Button text="Register" onClick={handleSubmit} />
        </FormBox>
      </div>
    </div>
  );
}

export default LockersRegistration;
